function wackyPatch(parentObj, name, patches) {
	if (typeof parentObj === "string") {
		//assume parentObj and name are switched around (for backcompat/convienence)
		const tmp = parentObj;
		parentObj = name;
		name = tmp;
	}
	const injId = Symbol();
	const before = patches["before"];
	const instead = patches["instead"];
	const after = patches["after"];
	const handler = {
		apply: (target, thisArg, [ctx, args]) => {
			if (before !== undefined) before.apply(ctx, [args]);
			const res =
				patches["instead"] !== undefined
					? instead.apply(ctx, [args, target.bind(ctx)])
					: target.apply(ctx, args);
			if (after === undefined) return res;
			return after.apply(ctx, [args, res]);
		}
	};
	const prox = new Proxy(parentObj[name], handler);
	const orig = parentObj[name];
	parentObj[name] = function () {
		return prox(this, arguments);
	};
	const unpatch = () => {
		parentObj[name] = orig;
	};
	parentObj[injId] = {
		name: name,
		orig: orig,
		unpatch: unpatch
	};
	return unpatch;
}

function before(parentObj, name, func) {
	return wackyPatch(parentObj, name, { before: func });
}

function instead(parentObj, name, func) {
	return wackyPatch(parentObj, name, { instead: func });
}

function after(parentObj, name, func) {
	return wackyPatch(parentObj, name, { after: func });
}

export { instead, before, after };

export default {
	instead,
	before,
	after
};
